/* (c) 2004 Matthew Garrett, almost entirely based on read-edid */
/* (c) 2000,2001 John Fremlin */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <sys/io.h>

#include "lrmi.h"

typedef u_int8_t byte;
typedef u_int32_t uint32;

typedef struct LRMI_regs reg_frame;
typedef byte* real_ptr;

#define access_register(reg_frame,reg) (reg_frame . reg)
#define access_ptr_register(reg_frame,reg) (reg_frame -> reg)
#define access_seg_register(reg_frame,es) reg_frame.es
#define real_mode_int(interrupt,reg_frame_ptr) !LRMI_int(interrupt,reg_frame_ptr)
#define alloc_real(size) LRMI_alloc_real(size)
#define free_real(block) LRMI_free_real(block)
#define dosmemput(buffer,length,offset) memcpy(offset,buffer,length)

#define DPMS_STATE_ON 0x0000
#define DPMS_STATE_STANDBY 0x0100
#define DPMS_STATE_SUSPEND 0x0200
#define DPMS_STATE_OFF 0x0400
#define DPMS_STATE_LOW 0x0800

int do_blank(int mode);

real_ptr far_ptr_to_real_ptr( uint32 farptr )
{
  unsigned segment = (farptr & 0xffff0000)>>16;
  unsigned offset = (farptr & 0xffff);
  return (real_ptr)(segment*16+offset);
}

const char* supported_message[] =
  {
    "Function unsupported",
    "Function supported"
  }
;
const char* call_successful_message[] =
  {
    "Call failed",
    "Call successful"
  }
;

int
main( int argc, char** argv )
{
	int error=0;
	
	if( !LRMI_init() )
	{
		fprintf( stderr, "%s: error initialising realmode interface\n", argv[0] );
		fprintf( stderr, "%s: do you have full superuser (root) permissions?\n", argv[0] );
		return 10;
	}
	
	ioperm(0, 0x400 , 1);
	iopl(3);
	
	if ( argc != 2) {
		fprintf( stderr, "%s: syntax %s [on|suspend|standby|off|reduced\n",
			 argv[0], argv[0]);
		return 3;
	} else {
		if (!strcmp(argv[1],"on")) {
			do_blank(DPMS_STATE_ON);
		} else if (!strcmp(argv[1],"suspend")) {
			do_blank(DPMS_STATE_SUSPEND);
		} else if (!strcmp(argv[1],"standby")) {
			do_blank(DPMS_STATE_STANDBY);
		} else if (!strcmp(argv[1],"off")) {
			do_blank(DPMS_STATE_OFF);
		} else if (!strcmp(argv[1],"reduced")) {
			do_blank(DPMS_STATE_LOW);
		} else {
			fprintf( stderr, "%s: syntax %s [on|suspend|standby|off|reduced\n",
				 argv[0], argv[0]);
			return 4;
		}
	}
	
	return error;
}

int do_vbe_service(unsigned AX,unsigned BX,reg_frame* regs)
{
  const unsigned interrupt = 0x10;
  unsigned function_sup;
  unsigned success;
  int error = 0;

  access_ptr_register(regs,eax) = AX;
  access_ptr_register(regs,ebx) = BX;

  if( real_mode_int(interrupt, regs) )
    {
      fprintf( stderr, "Error: something went wrong performing real mode interrupt\n" );
      error = 1;
    }
  
  AX = access_ptr_register(regs,eax);

  function_sup = ((AX & 0xff) == 0x4f);
  success = ((AX & 0xff00) == 0);
  
  if (!success)
    error=1;
  if (!function_sup)
    error=2;
  
  return error;
}
  
int
do_blank(int state)
{
  reg_frame regs;
  memset(&regs, 0, sizeof(regs));
  return do_vbe_service(0x4f10,state|=0x01,&regs);
}
